#include "lcd.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


#define TOP_LINE 0x00
#define BOTTOM_LINE 0x40
#define EMPTY_LINE "                "

#define PRESCALER 256

#define E  164.81
#define A  220.00
#define D  293.66
#define G  391.99
#define B  493.88
#define EH 659.26

float standard_strings[] = {82.41, 110.0, 146.83, 196.0, 246.94, 329.63};
char *str_standard_strings[] = {"e ", "A  ", "D  ", "G  ", "B ", "E"};
char chosen_string = 0;



/* Prescaler set to 256 */
void timer1_init (void)
{
    TCCR1B |= (1 << CS12) | (1 << WGM12);
    TIMSK1 |= ( 1<< OCIE1A);
    OCR1A = 31250; /* TOP */
}

void LCD_clear (void)
{
    LCD_printAt (TOP_LINE, EMPTY_LINE);
    LCD_printAt (BOTTOM_LINE, EMPTY_LINE);
}


void adc_init (void)
{
    // Input pin PA0
    DDRA &= ~(1 << PA0);
    ADMUX |= (3 << REFS0);
    ADCSRA |= (1 << ADEN) | (5 << ADPS0);
}

float adc_read()
{
    ADCSRA |= (1 << ADSC);
    ADMUX = (ADMUX & 0xF8) | 1; // clearing the bits, then ORing
    while(ADCSRA & (1 << ADSC));

    return (ADC);
}

void wait_for (int millisecs)
{
    int i;
    for (i = 0; i < millisecs; i++)
    {
        _delay_ms (1);
    }
}

void refresh_bottom_line (void)
{
    LCD_printAt (BOTTOM_LINE, EMPTY_LINE);
}

void startup_LCD (void)
{
    int i;

    LCD_printAt (TOP_LINE, "Paul Dumitriu");
    LCD_printAt (BOTTOM_LINE, "Guitar Tuner");
    wait_for (2000);
    LCD_clear ();

    /* Display available strings. */
    char *strings = strdup ("");
    for (i = 0; i < 6; i++) {
        strcat (strings, str_standard_strings[i]);
    }
    LCD_printAt (TOP_LINE, strings);

    /* Display cursors below available strings. */
    refresh_bottom_line ();
    LCD_printAt (TOP_LINE, "");
}

int main (void) {
    int i;
    /* Init LCD and show welcome message. */
    LCD_init ();
    startup_LCD ();

    /* Init input and timer. */
    adc_init ();
    timer1_init ();
    sei();

    /* Infinite loop to run program. */
    for (;;) {
        float temp_diff;
        float min = 9999;
        int sign = 0;
        int current_string = 0;
        float adc_val = adc_read();
        LCD_clear();
        for(i = 0; i < 8; i++) {
            temp_diff = adc_val - standard_strings[i];
            if((temp_diff < 0 && -temp_diff < min )|| (temp_diff >= 0 && temp_diff < min))
            {
                if(temp_diff < 0)
                {
                    min = -temp_diff;
                    sign = -1;
                }
                else
                {
                    min = temp_diff;
                    sign = 1;
                }
                current_string = i;
            }
        }
        LCD_printAt(TOP_LINE, str_standard_strings[current_string]);
        if(min < 1)
        {
            LCD_printAt(BOTTOM_LINE, "OK");
        }
        else
        {
            if(sign < 0)
            {
                LCD_printAt(BOTTOM_LINE, ">>");
            }
            else
            {
                LCD_printAt(BOTTOM_LINE, "<<");
            }
        }
        wait_for(1000);
    }

    return 0;
}



